@@ -1,17 +1,18 @@
 import importlib.util
-import sys
-import time
-
 import logging.config
 import os
 import re
 import signal
-import yaml
-from IPy import IP
-from dotted.collection import DottedDict
+import sys
+import time
 from pathlib import Path
 from typing import Any, List, Optional
 
+import yaml
+from IPy import IP
+
+from irrd.vendor.dotted.collection import DottedDict
+
 CONFIG_PATH_DEFAULT = '/etc/irrd.yaml'
 
 logger = logging.getLogger(__name__)
@@ -19,6 +20,87 @@ PASSWORD_HASH_DUMMY_VALUE = 'DummyValue'
 SOURCE_NAME_RE = re.compile('^[A-Z][A-Z0-9-]*[A-Z0-9]$')
 RPKI_IRR_PSEUDO_SOURCE = 'RPKI'
 
+# Note that sources are checked separately,
+# and 'access_lists' is always permitted
+KNOWN_CONFIG_KEYS = DottedDict({
+    'database_url': {},
+    'database_readonly': {},
+    'redis_url': {},
+    'piddir': {},
+    'user': {},
+    'group': {},
+    'server': {
+        'http': {
+            'interface': {},
+            'port': {},
+            'status_access_list': {},
+            'workers': {},
+            'forwarded_allowed_ips': {},
+        },
+        'whois': {
+            'interface': {},
+            'port': {},
+            'access_list': {},
+            'max_connections': {},
+        },
+    },
+    'email': {
+        'from': {},
+        'footer': {},
+        'smtp': {},
+        'recipient_override': {},
+        'notification_header': {},
+    },
+    'auth': {
+        'override_password': {},
+        'authenticate_related_mntners': {},
+        'gnupg_keyring': {},
+    },
+    'rpki': {
+        'roa_source': {},
+        'roa_import_timer': {},
+        'slurm_source': {},
+        'pseudo_irr_remarks': {},
+        'notify_invalid_enabled': {},
+        'notify_invalid_subject': {},
+        'notify_invalid_header': {},
+    },
+    'scopefilter': {
+        'prefixes': {},
+        'asns': {},
+    },
+    'log': {
+        'logfile_path': {},
+        'level': {},
+        'logging_config_path': {},
+    },
+    'sources_default': {},
+    'compatibility': {
+        'inetnum_search_disabled': {},
+        'irrd42_migration_in_progress': {},
+        'permit_non_hierarchical_as_set_name': {},
+        'ipv4_only_route_set_members': {},
+    }
+})
+
+KNOWN_SOURCES_KEYS = {
+    'authoritative',
+    'keep_journal',
+    'nrtm_host',
+    'nrtm_port',
+    'import_source',
+    'import_serial_source',
+    'import_timer',
+    'object_class_filter',
+    'export_destination',
+    'export_destination_unfiltered',
+    'export_timer',
+    'nrtm_access_list',
+    'nrtm_access_list_unfiltered',
+    'strict_import_keycert_objects',
+    'rpki_excluded',
+    'scopefilter_excluded',
+}
 
 LOGGING = {
     'version': 1,
@@ -64,7 +146,7 @@ class ConfigurationError(ValueError):
 # testing_overrides can be set to a DottedDict, and is used
 # to override settings while testing, using the config_override
 # fixture.
-testing_overrides = None
+testing_overrides: Any = None
 
 
 class Configuration:
@@ -84,6 +166,7 @@ class Configuration:
         default_config_path = str(Path(__file__).resolve().parents[0] / 'default_config.yaml')
         default_config_yaml = yaml.safe_load(open(default_config_path))
         self.default_config = DottedDict(default_config_yaml['irrd'])
+        self.logging_config = LOGGING
 
         errors = self._staging_reload_check(log_success=False)
         if errors:
@@ -96,7 +179,7 @@ class Configuration:
             logfile_path = self.get_setting_live('log.logfile_path')
             if logging_config_path:
                 spec = importlib.util.spec_from_file_location("logging_config", logging_config_path)
-                config_module = importlib.util.module_from_spec(spec)
+                config_module = importlib.util.module_from_spec(spec)  # type: ignore
                 spec.loader.exec_module(config_module)  # type: ignore
                 self.logging_config = config_module.LOGGING  # type: ignore
                 logging.config.dictConfig(self.logging_config)
@@ -126,6 +209,14 @@ class Configuration:
         If it is not found in any, the value of the default paramater
         is returned, which is None by default.
         """
+        if setting_name.startswith('sources'):
+            components = setting_name.split('.')
+            if len(components) == 3 and components[2] not in KNOWN_SOURCES_KEYS:
+                raise ValueError(f'Unknown setting {setting_name}')
+        elif not setting_name.startswith('access_lists'):
+            if KNOWN_CONFIG_KEYS.get(setting_name) is None:
+                raise ValueError(f'Unknown setting {setting_name}')
+
         env_key = 'IRRD_' + setting_name.upper().replace('.', '_')
         if env_key in os.environ:
             return os.environ[env_key]
@@ -198,6 +289,19 @@ class Configuration:
         errors = []
         config = self.user_config_staging
 
+        for key, value in config.items():
+            if key in ['sources', 'access_lists']:
+                continue
+            known = KNOWN_CONFIG_KEYS.get(key)
+            if known is None:
+                errors.append(f'Unknown setting key: {key}')
+            if hasattr(value, 'items'):
+                for key2, value2 in value.items():
+                    subkey = key + '.' + key2
+                    known_sub = KNOWN_CONFIG_KEYS.get(subkey)
+                    if known_sub is None:
+                        errors.append(f'Unknown setting key: {subkey}')
+
         if not self._check_is_str(config, 'database_url'):
             errors.append('Setting database_url is required.')
 
@@ -209,10 +313,10 @@ class Configuration:
 
         expected_access_lists = {
             config.get('server.whois.access_list'),
-            config.get('server.http.access_list'),
+            config.get('server.http.status_access_list'),
         }
 
-        if not self._check_is_str(config, 'email.from') or '@' not in config.get('email.from'):
+        if not self._check_is_str(config, 'email.from') or '@' not in config.get('email.from', ''):
             errors.append('Setting email.from is required and must be an email address.')
         if not self._check_is_str(config, 'email.smtp'):
             errors.append('Setting email.smtp is required.')
@@ -220,21 +324,19 @@ class Configuration:
                 or '@' not in config.get('email.recipient_override', '@'):
             errors.append('Setting email.recipient_override must be an email address if set.')
 
-        string_not_required = ['email.footer', 'server.whois.access_list', 'server.http.access_list',
-                               'rpki.notify_invalid_subject', 'rpki.notify_invalid_header',
-                               'rpki.slurm_source']
+        string_not_required = ['email.footer', 'server.whois.access_list',
+                               'server.http.status_access_list', 'rpki.notify_invalid_subject',
+                               'rpki.notify_invalid_header', 'rpki.slurm_source', 'user', 'group']
         for setting in string_not_required:
             if not self._check_is_str(config, setting, required=False):
                 errors.append(f'Setting {setting} must be a string, if defined.')
 
+        if bool(config.get('user')) != bool(config.get('group')):
+            errors.append('Settings user and group must both be defined, or neither.')
+
         if not self._check_is_str(config, 'auth.gnupg_keyring'):
             errors.append('Setting auth.gnupg_keyring is required.')
 
-        access_lists = set(config.get('access_lists', {}).keys())
-        unresolved_access_lists = {x for x in expected_access_lists.difference(access_lists) if x and isinstance(x, str)}
-        if unresolved_access_lists:
-            errors.append(f'Access lists {", ".join(unresolved_access_lists)} referenced in settings, but not defined.')
-
         for name, access_list in config.get('access_lists', {}).items():
             for item in access_list:
                 try:
@@ -263,6 +365,9 @@ class Configuration:
 
         has_authoritative_sources = False
         for name, details in config.get('sources', {}).items():
+            unknown_keys = set(details.keys()) - KNOWN_SOURCES_KEYS
+            if unknown_keys:
+                errors.append(f'Unknown key(s) under source {name}: {", ".join(unknown_keys)}')
             if details.get('authoritative'):
                 has_authoritative_sources = True
             if config.get('rpki.roa_source') and name == RPKI_IRR_PSEUDO_SOURCE:
@@ -282,6 +387,10 @@ class Configuration:
                 errors.append(f'Setting authoritative for source {name} can not be enabled when either '
                               f'nrtm_host or import_source are set.')
 
+            if config.get('database_readonly') and (details.get('authoritative') or details.get('nrtm_host') or details.get('import_source')):
+                errors.append(f'Source {name} can not have authoritative, import_source or nrtm_host set '
+                              f'when database_readonly is enabled.')
+
             if not str(details.get('nrtm_port', '43')).isnumeric():
                 errors.append(f'Setting nrtm_port for source {name} must be a number.')
             if not str(details.get('import_timer', '0')).isnumeric():
@@ -289,6 +398,11 @@ class Configuration:
             if not str(details.get('export_timer', '0')).isnumeric():
                 errors.append(f'Setting export_timer for source {name} must be a number.')
 
+            if details.get('nrtm_access_list'):
+                expected_access_lists.add(details.get('nrtm_access_list'))
+            if details.get('nrtm_access_list_unfiltered'):
+                expected_access_lists.add(details.get('nrtm_access_list_unfiltered'))
+
         if config.get('rpki.roa_source', 'https://rpki.gin.ntt.net/api/export.json'):
             known_sources.add(RPKI_IRR_PSEUDO_SOURCE)
             if has_authoritative_sources and config.get('rpki.notify_invalid_enabled') is None:
@@ -313,6 +427,12 @@ class Configuration:
             errors.append('Setting log.logging_config_path can not be combined with'
                           'log.logfile_path or log.level')
 
+        access_lists = set(config.get('access_lists', {}).keys())
+        unresolved_access_lists = [x for x in expected_access_lists.difference(access_lists) if x and isinstance(x, str)]
+        unresolved_access_lists.sort()
+        if unresolved_access_lists:
+            errors.append(f'Access lists {", ".join(unresolved_access_lists)} referenced in settings, but not defined.')
+
         return errors
 
     def _check_is_str(self, config, key, required=True):
@@ -321,7 +441,7 @@ class Configuration:
         return config.get(key) is None or isinstance(config.get(key), str)
 
 
-configuration = None
+configuration: Optional[Configuration] = None
 
 
 def get_configuration() -> Optional[Configuration]:
