@@ -1,4 +1,5 @@
 import datetime
+import itertools
 import json
 import re
 from collections import OrderedDict, Counter
@@ -35,6 +36,9 @@ class RPSLObjectMeta(type):
             cls.attrs_allowed = [field[0] for field in fields.items()]
             cls.attrs_required = [field[0] for field in fields.items() if not field[1].optional]
             cls.attrs_multiple = [field[0] for field in fields.items() if field[1].multiple]
+            cls.field_extracts = list(itertools.chain(
+                *[field[1].extracts for field in fields.items() if field[1].primary_key or field[1].lookup_key]
+            ))
             cls.referring_strong_fields = [(field[0], field[1].referring) for field in fields.items() if hasattr(field[1], 'referring') and getattr(field[1], 'strong')]
 
 
@@ -60,7 +64,7 @@ class RPSLObject(metaclass=RPSLObjectMeta):
     ip_last: IP = None
     asn_first: Optional[int] = None
     asn_last: Optional[int] = None
-    prefix: IP = None  # Note: not saved to DB, used for performance
+    prefix: IP = None
     prefix_length: Optional[int] = None
     rpki_status: RPKIStatus = RPKIStatus.not_found
     scopefilter_status: ScopeFilterStatus = ScopeFilterStatus.in_scope
@@ -70,6 +74,9 @@ class RPSLObject(metaclass=RPSLObjectMeta):
     rpki_relevant = False
     # Fields whose values are discarded during parsing
     discarded_fields: List[str] = []
+    # Fields that are ignored in validation even
+    # for authoritative objects (see #587 for example).
+    ignored_validation_fields: List[str] = ['last-modified']
 
     _re_attr_name = re.compile(r'^[a-z0-9_-]+$')
 
@@ -212,6 +219,15 @@ class RPSLObject(metaclass=RPSLObjectMeta):
         return template
 
     def clean(self) -> bool:
+        """
+        Additional cleaning steps for some objects.
+        """
+        return True
+
+    def clean_for_create(self) -> bool:
+        """
+        Additional cleaning steps for creations only.
+        """
         return True
 
     def _extract_attributes_values(self, text: str) -> None:
@@ -287,6 +303,8 @@ class RPSLObject(metaclass=RPSLObjectMeta):
 
         if self.strict_validation:
             for attr_name, count in attrs_present.items():
+                if attr_name in self.ignored_validation_fields:
+                    continue
                 if attr_name not in self.attrs_allowed:
                     self.messages.error(f'Unrecognised attribute {attr_name} on object {self.rpsl_object_class}')
                 if count > 1 and attr_name not in self.attrs_multiple:
@@ -445,7 +463,7 @@ class RPSLObject(metaclass=RPSLObjectMeta):
         return isinstance(self, type(other)) and self.__key() == other.__key()
 
 
-class UnknownRPSLObjectClassException(Exception):
+class UnknownRPSLObjectClassException(Exception):  # noqa: N818
     def __init__(self, message: str, rpsl_object_class: str) -> None:
         self.message = message
         self.rpsl_object_class = rpsl_object_class
