@@ -16,4 +16,6 @@ class UpdateRequestStatus(Enum):
     ERROR_PARSING = 'errors encountered during object parsing'
     ERROR_AUTH = 'error: update not authorised'
     ERROR_REFERENCE = 'error: reference to object that does not exist'
+    ERROR_ROA = 'error: conflict with existing ROA'
+    ERROR_SCOPEFILTER = 'error: not in scope'
     ERROR_NON_AUTHORITIVE = 'error: attempt to update object in non-authoritive database'
