@@ -12,7 +12,7 @@ source:         TEST
 remarks:        remark
 """
 
-SAMPLE_AS_SET = """as-set:         AS-SETTEST
+SAMPLE_AS_SET = """as-set:         AS65537:AS-SETTEST
 descr:          description
 members:        AS65538, AS65539
 members:        AS65537
@@ -207,6 +207,7 @@ netname:        NET-TEST-V4
 descr:          description
 country:        IT
 notify:         notify@example.com
+geofeed:        https://example.com/geofeed
 admin-c:        PERSON-TEST
 tech-c:         PERSON-TEST
 status:         ASSIGNED PA
